# -*- coding: utf-8 -*-
import sys, json, re, time
from bs4 import BeautifulSoup
try:
    from urllib2 import urlopen, Request
except ImportError:
    from urllib.request import urlopen, Request
try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote
try:
    from urlparse import urlparse, urljoin
except ImportError:
    from urllib.parse import urlparse, urljoin


def getsoup(content):
    if (2, 7, 3) <= sys.version_info < (3, 0) or (3, 2) <= sys.version_info < (4, 0):
        return BeautifulSoup(content, 'html.parser')
    return BeautifulSoup(content, 'html5lib')


def httpget(url, referer=None):
    request = Request(url)
    if referer:
        request.add_header('Referer', referer)
    return urlopen(request, timeout=10).read().decode('utf-8')


def show_page(url):
    o = urlparse(url)
    id = re.search(r'/show_page/id_(.*?)\.html', o.path)
    if not id:
        return {'code': 'youku 200'}
    id = id.group(1)
    url = 'http://list.youku.com/show/id_%s.html' % id
    return list_show_id(url)


def list_show_id(url):
    o = urlparse(url)
    id = re.search(r'/show/id_(.*?)\.html', o.path)
    if not id:
        return {'code': 'youku 200'}
    id = id.group(1)
    url = 'http://list.youku.com/show/id_%s.html' % id
    try:
        content = urlopen(url, timeout=10).read().decode('utf-8')
    except Exception as e:
        return {'code': 'youku 201', 'error': str(e)}
    showid = re.search(r'showid:"(.*?)"', content)
    if not showid:
        return {'code': 'youku 202'}
    showid = showid.group(1)
    # url = 'http://list.youku.com/show/module?id=%s&tab=showInfo&callback=callback' % showid
    url = 'http://list.youku.com/show/module?id=%s&tab=point&callback=callback' % showid
    try:
        content = urlopen(url, timeout=10).read().decode('utf-8')
    except Exception as e:
        return {'code': 'youku 203', 'error': str(e)}
    content = re.search(r'\((.*)\)', content)
    if not content:
        return {'code': 'youku 204'}
    content = content.group(1)
    try:
        data = json.loads(content)
    except Exception as e:
        return {'code': 'youku 205', 'error': str(e)}
    try:
        soup = getsoup(data['html'])
    except Exception as e:
        return {'code': 'youku 206', 'error': str(e)}
    ret = []
    for e in soup.select('ul li[data-id]'):
        url1 = 'http://list.youku.com/show/point?id=%s&stage=%s&callback=callback' % (showid, e['data-id'])
        ret.append({'title': e.text, 'url': url1})
    if ret:
        return ret
    for e in soup.select('div.p-drama-list div.p-item'):
        e1 = e.select('div.item-title a')[0]
        title = e1.text
        img = e.select('img')[0]['src']
        plot = '%s\n%s' % (e.select('span.p-time')[0].text, e.select('div.item-intro')[0].text)
        ret.append({'title': title, 'url': urljoin(url, e1['href']), 'img': img, 'plot': plot})
    return ret


def list_show_point(url):
    try:
        content = urlopen(url, timeout=10).read().decode('utf-8')
    except Exception as e:
        return {'code': 'youku 220', 'error': str(e)}
    content = re.search(r'\((.*)\)', content)
    if not content:
        return {'code': 'youku 221'}
    content = content.group(1)
    try:
        data = json.loads(content)
    except Exception as e:
        return {'code': 'youku 222', 'error': str(e)}
    try:
        soup = getsoup(data['html'])
    except Exception as e:
        return {'code': 'youku 223', 'error': str(e)}
    ret = []
    for e in soup.select('div.p-drama-list div.p-item'):
        e1 = e.select('div.item-title a')[0]
        title = e1.text
        img = e.select('img')[0]['src']
        plot = '%s\n%s' % (e.select('span.p-time')[0].text, e.select('div.item-intro')[0].text)
        ret.append({'title': title, 'url': urljoin(url, e1['href']), 'img': img, 'plot': plot})
    return ret


def show_episode(url):
    try:
        content = urlopen(url, timeout=10).read().decode('utf-8')
    except Exception as e:
        return {'code': 'youku 300', 'error': str(e)}
    try:
        soup = getsoup(content)
    except Exception as e:
        return {'code': 'youku 301', 'error': str(e)}
    ret = []
    for a in soup.select('ul li a'):
        if 'href' not in a.attrs:
            continue
        ret.append({'title': a.text, 'url': a['href']})
    return ret


def playlist_show(url):
    try:
        content = urlopen(url, timeout=10).read().decode('utf-8')
    except Exception as e:
        return {'code': 'youku 400', 'error': str(e)}
    try:
        soup = getsoup(content)
    except Exception as e:
        return {'code': 'youku 401', 'error': str(e)}
    ret = []
    for a in soup.select('ul.v a'):
        if 'title' not in a.attrs or 'href' not in a.attrs:
            continue
        ret.append({'title': a['title'], 'url': a['href']})
    e = soup.find('div', {'class': ['page', 'f_r']})
    if e:
        a = e.find('a')
        if a and 'href' in a.attrs and 'title' in a.attrs and a['title'] == '下一页':
            ret.append({'title': 'Load more', 'url': a['href']})
    return ret


def play(url):
    def quote_cna(cna):
        if '%' in cna:
            return cna
        return quote(cna)

    def fetch_cna():
        # return 'FwxJFKUKpkoCAW6vuatKK0fS'
        url = 'http://log.mmstat.com/eg.js'
        try:
            req = urlopen(url)
        except e:
            raise e
        for header in req.info():
            if header.lower() == 'set-cookie':
                n_v = req.info()[header].split(';')[0]
                name, value = n_v.split('=')
                if name == 'cna':
                    return quote_cna(value)
        return quote_cna('FwxJFKUKpkoCAW6vuatKK0fS')

    o = urlparse(url)
    if o.netloc != 'v.youku.com':
        return {'code': 'youku 100'}
    vid = re.search(r'/v_show/id_(.*?)\.html', o.path)
    if not vid:
        return {'code': 'youku 100'}
    vid = vid.group(1)
    try:
        test = fetch_cna()
    except Exception as e:
        return {'code': 'youku 101', 'error': str(e)}
    # ccode = '0524'
    for i in range(524, 525):
        ccode = '%04d' % i
        url = 'https://ups.youku.com/ups/get.json?vid=%s&ccode=%s' % (vid, ccode)
        url += '&client_ip=192.168.1.1'
        url += '&utid=' + test
        url += '&client_ts=' + str(int(time.time()))
        try:
            content = httpget(url, referer='https://v.youku.com')
        except Exception as e:
            return {'code': 'youku 102', 'error': str(e)}
        try:
            data = json.loads(content)
        except Exception as e:
            return {'code': 'youku 103', 'error': str(e)}
        try:
            urls = []
            for stream in data['data']['stream']:
                if stream['segs']:
                    urls2 = []
                    for seg in stream['segs']:
                        urls2.append(seg['cdn_url'])
                    urls.append({'title': stream['stream_type'], 'url': urls2})
        except:
            try:
                return {'code': data['data']['error']['note']}
            except Exception as e:
                return {'code': 'youku 104', 'error': str(e)}
        return urls
    # return {'title': data['data']['video']['title'], 'url': urls}


def ykdl(url):
    import sys
    reload(sys)
    sys.setdefaultencoding('utf-8')
    from ykdl.common import url_to_module
    m, u = url_to_module(url)
    parser = m.parser
    parser(u)
    info = m.prepare()
    info = info.jsonlize()
    ret = []
    for k, v in info['streams'].items():
        ret.append({'title': k, 'url': v['src']})
    return ret
